/*
Copyright (C) 2005 Manoel Kasimier ( manoelkasimier@yahoo.com.br )
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <kos.h>
#include "quakedef.h"

extern int DC_CheckDrive();
static qboolean cd_paused;
static qboolean inited = false;
static float cdvolume = 0;

void CDAudio_Play(byte track, qboolean looping)
{
	int trnum = track;

	if(!inited || !cd_enabled)
		return;

	// Ignore if playing the same track
	if (DC_CheckDrive() == CD_STATUS_PLAYING && trnum == playTrack)
		return;

	// Stop old track
	CDAudio_Stop();

	// Play new track
	if (!cdrom_cdda_play(trnum, trnum, looping*15, CDDA_TRACKS))
	{
		playTrack = trnum;
		playLooping = looping;
	}
}

void CDAudio_Stop(void)
{
	CDAudio_Pause(); // Just pause, there's no need to spin down
	playTrack = 0; // Fake stop
	cd_paused = false;
}

void CDAudio_Pause(void)
{
	if(!inited || !cd_enabled)
		return;
	if (DC_CheckDrive() == CD_STATUS_PLAYING)
	{
		cdrom_cdda_pause();
		cd_paused = true;
	}
}

void CDAudio_Resume(void)
{
	if(!inited || !cd_enabled)
		return;
	if (DC_CheckDrive() == CD_STATUS_PAUSED && playTrack)
	{
		cdrom_cdda_resume();
		cd_paused = false;
	}
}

void CDAudio_Update(void)
{
	if(!inited || !cd_enabled)
		return;

	// Has the volume level changed?
	if (bgmvolume.value != cdvolume)
	{
		int newvol;
		cdvolume = bgmvolume.value;

		// Work it out as an integer, clip to limits
		newvol = cdvolume * 256;
		if(newvol > 255)
			newvol = 255;
		else if(newvol < 0)
			newvol = 0;

		// Reset volume
		// TO DO: how to do this in KOS?
	}
	if (playTrack)
		if (DC_CheckDrive() != CD_STATUS_PLAYING && cd_paused == false)
			// the GD-ROM drive probably stopped to load something, so let's restart the music
			if (cdrom_cdda_play((int)playTrack, (int)playTrack, playLooping*15, CDDA_TRACKS))
				Con_Printf("CDAudio_Update: failed to restart track %u\n", playTrack);
}

static void CD_f (void)
{
	char	*command;
	command = Cmd_Argv(1);

	if (Cmd_Argc() < 2)
		return;

	if (!Q_strcasecmp(command, "off"))
	{
		CDAudio_Stop();
		cd_enabled = false;
	}
	if (!Q_strcasecmp(command, "on"))		cd_enabled = true;
	if (!Q_strcasecmp(command, "play"))		CDAudio_Play((byte)Q_atoi(Cmd_Argv (2)), false);
	if (!Q_strcasecmp(command, "loop"))		CDAudio_Play((byte)Q_atoi(Cmd_Argv (2)), true);
	if (!Q_strcasecmp(command, "stop"))		CDAudio_Stop();
	if (!Q_strcasecmp(command, "pause"))	CDAudio_Pause();
	if (!Q_strcasecmp(command, "resume"))	CDAudio_Resume();
	if (!Q_strcasecmp(command, "info"))
	{
		if (DC_CheckDrive() == CD_STATUS_PLAYING)
			Con_Printf("Currently %s track %u\n", playLooping ? "looping" : "playing", playTrack);
		else if (DC_CheckDrive() == CD_STATUS_PAUSED)
			Con_Printf("Currently paused track %u\n", playTrack);
		Con_Printf("Volume is %f\n", cdvolume);
		return;
	}
}

int CDAudio_Init(void)
{
	// Manoel Kasimier - begin
	maxTrack = 99;
	cdValid = true; // duh
	// Manoel Kasimier - end
	inited = cd_enabled = true;
	CDAudio_Update();
	Cmd_AddCommand ("cd", CD_f);
	return 0;
}

void CDAudio_Shutdown(void)
{
	inited = cd_enabled = false;
	cdrom_spin_down();
}
