/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <kos.h>
#include <oggvorbis/sndoggvorbis.h>
#include "quakedef.h"

static qboolean inited = false;
//static qboolean cd_enabled = false;
static float cdvolume = 0;
//static int playTrack = 0; // Manoel Kasimier - removed
//static qboolean playLooping = 0; // Manoel Kasimier - removed

int COM_FindFilePath(char *filename, char *path);

void CDAudio_Play(byte track, qboolean looping)
{
	char filename[MAX_OSPATH];
	char path[MAX_OSPATH];
	int trnum = track;

	// Don't bother if it's disabled
	if((!inited)||(!cd_enabled))
		return;

	// Stop whatever's playing right now
	CDAudio_Stop();

	// Try to find the appropriate file
	sprintf(filename, "music/track%i%i.ogg", track/10, track%10); // Manoel Kasimier
//	sprintf(filename, "music/track%d.ogg", trnum);
	if(!COM_FindFilePath(filename, path))
		return;

	// Start playing it
	sndoggvorbis_start(path, looping);
	playTrack = trnum;
	playLooping = looping;
}

void CDAudio_Stop(void)
{
	// Don't bother if it's disabled
	if((!inited)||(!cd_enabled))
		return;

	// It it's playing, stop it
	if(sndoggvorbis_isplaying())
		sndoggvorbis_stop();
}

void CDAudio_Pause(void)
{
	// Don't bother if it's disabled
	if((!inited)||(!cd_enabled))
		return;

	// It it's playing, stop it
	if(sndoggvorbis_isplaying())
		sndoggvorbis_stop();
}

void CDAudio_Resume(void)
{
	// Don't bother if it's disabled
	if((!inited)||(!cd_enabled))
		return;
}

void CDAudio_Update(void)
{
	// Don't bother if it's disabled
	if((!inited)||(!cd_enabled))
		return;

	// Has the volume level changed?
	if (bgmvolume.value != cdvolume)
	{
		int newvol;
		cdvolume = bgmvolume.value;

		// Work it out as an integer, clip to limits
		newvol = cdvolume * 256;
		if(newvol > 255)
			newvol = 255;
		else if(newvol < 0)
			newvol = 0;

		// Reset volume
		sndoggvorbis_volume(newvol);
	}
}

static void CD_f (void)
{
	char	*command;

	if (Cmd_Argc() < 2)
		return;

	command = Cmd_Argv(1);

	if (Q_strcasecmp(command, "on") == 0)
	{
		cd_enabled = true;
		return;
	}

	if (Q_strcasecmp(command, "off") == 0)
	{
		if (sndoggvorbis_isplaying())
			CDAudio_Stop();
		cd_enabled = false;
		return;
	}

	if (Q_strcasecmp(command, "reset") == 0)
	{
		Con_Printf("CD RESET not supported\n");
	}

	if (Q_strcasecmp(command, "remap") == 0)
	{
		Con_Printf("CD REMAP not supported\n");
	}

	if (Q_strcasecmp(command, "close") == 0)
	{
		Con_Printf("CD CLOSE not supported\n");
		return;
	}

	if (Q_strcasecmp(command, "play") == 0)
	{
		CDAudio_Play((byte)Q_atoi(Cmd_Argv (2)), false);
		return;
	}

	if (Q_strcasecmp(command, "loop") == 0)
	{
		CDAudio_Play((byte)Q_atoi(Cmd_Argv (2)), true);
		return;
	}

	if (Q_strcasecmp(command, "stop") == 0)
	{
		CDAudio_Stop();
		return;
	}

	if (Q_strcasecmp(command, "pause") == 0)
	{
		CDAudio_Pause();
		return;
	}

	if (Q_strcasecmp(command, "resume") == 0)
	{
		CDAudio_Resume();
		return;
	}

	if (Q_strcasecmp(command, "eject") == 0)
	{
		Con_Printf("CD EJECT not supported\n");
	}

	if (Q_strcasecmp(command, "info") == 0)
	{
		if(sndoggvorbis_isplaying())
			Con_Printf("Currently %s track %u\n", playLooping ? "looping" : "playing", playTrack);
		Con_Printf("Volume is %f\n", cdvolume);
		return;
	}
}

int	disableogg=true; // Manoel Kasimier - hack
int CDAudio_Init(void)
{
	// Manoel Kasimier - begin
	char	filename[MAX_OSPATH];
	char	path[MAX_OSPATH];
	int		track, skipped;

	if (disableogg) return -1; // Manoel Kasimier - hack

	skipped = maxTrack = 0;

	for (track=1; track<100; track++)
	{
		sprintf(filename, "music/track%i%i.ogg", track/10, track%10);
		if(!COM_FindFilePath(filename, path))
		{
			skipped++;
			continue;
		}
		maxTrack++;
	}
	if (maxTrack)
		cdValid = true;
	else
	{
		disableogg = true; // let SDL audio initialize if no .ogg tracks were found
		return -1;
	}
	maxTrack += skipped;
	// Manoel Kasimier - end
	// For the moment, this initializes the sound system too
	snd_stream_init();
	sndoggvorbis_init();
	inited = cd_enabled = true;
	CDAudio_Update();
	Cmd_AddCommand ("cd", CD_f);
	return 0;
}

void CDAudio_Shutdown(void)
{
	inited = cd_enabled = false;

	// Shut down Ogg Vorbis player and sound system
	sndoggvorbis_shutdown();
	snd_stream_shutdown();
}
